//spi_master_UNO
//
//先人の次のWebのプログラムをスレーブ専用に改造し搭載したESP32と通信する
//　Rabbit Note  ESP32 で SPI スレーブ通信するときの注意点
//    https://rabbit-note.com/2019/01/20/esp32-arduino-spi-slave/
//
// SPI通信の動作確認 
//   Master :  Arduino UNO     クロックスピード 16MHz  This sketch
//   Slave  :  ESP32 WROOM-32  クロックスピード240MHz
// Master側のスケッチ 
//ArduinoでSPI通信を行う方法 by Stupiddog 
// https://stupiddog.jp/note/archives/976
//
#include <SPI.h>

// 通信条件
static const uint32_t TRANS_SIZE =  512; // スレーブのバッファ長合わせる
SPISettings mySPISettings = SPISettings(8000000, MSBFIRST, SPI_MODE3);
//SCKの周波数:Arduino UNO(クロック周波数が16MHz)なので、8MHz、4MHz、2MHz、1MHz、
//など割り切れる周波数を指定する

// 通信バッファ
char buf[TRANS_SIZE];

//application data
char sendData[TRANS_SIZE];
uint32_t sendDataLength;
uint32_t recvDataLength;
uint32_t loopCounter = 0;

void print_buf(char* buf)
{
    char cha[256];
    for (uint32_t i = 0; i < TRANS_SIZE; i++) {
        cha[i] = buf[i];
        if(buf[i]==0) break;
    }
    Serial.println(cha);
}

void setup()
{
    Serial.begin(115200);
    delay(500);
    Serial.println("Serial Open");

    pinMode( SS, OUTPUT ); //デジタル10番ピンをOUTPUTに設定
    digitalWrite( SS, HIGH ); //デジタルの10番ピンからHighを出力
    
    // 送信データ作成
    for (uint32_t i = 0; i < TRANS_SIZE; i++) {
    sendData[i] = i & 0xFF;
    }
    sendDataLength = TRANS_SIZE;
    
    // SPI設定
    SPI.begin(); //デジタル10番がSS、11番がMOSI(Master Out Slave In)、12番がMISO、13番がSCK
}

void loop()
{
    Serial.print("\n[マスターLOOP ");
    Serial.print(loopCounter);
    Serial.println("]");
    loopCounter ++;

    Serial.print("TRANS_SIZE:"); Serial.print(TRANS_SIZE); Serial.print(" ");
    Serial.print("sendDataLength:"); Serial.println(sendDataLength);

    // 送るデータの準備
    for (uint32_t i = 0; i < sendDataLength; i++) {
        buf[i] = sendData[i];
    } 

    //トランザクションの開始
    SPI.beginTransaction(mySPISettings);
    digitalWrite(SS, LOW);
    SPI.transfer(buf,sendDataLength);
    // delay( 10 ); //必要に応じて設定
    digitalWrite(SS, HIGH);
    SPI.endTransaction();
    
    // チェック
    for (uint32_t i = 0; i < TRANS_SIZE; i++) {
        if(buf[i] == 0)
        {
          recvDataLength = i;
          break;
        }
    } 
    
    Serial.print("受信値：");  print_buf(buf);
    
    Serial.println("送信待ち");
    delay(5000);  
}
